import sys
import typing

from .puzzle_data_encryption import decrypt_data


def run_puzzle(
    in_puzzle: typing.Tuple[str, bytes],
    in_decrypt: typing.Callable[[bytes, bytes], bytes | None],
    get_answer,
):
    question, rest = in_puzzle
    print(question)
    if rest:
        this_pass = get_answer()
        new_puzzle = decrypt_data(rest, this_pass, in_decrypt)
        if new_puzzle is None:
            print("This is a wrong answer. Try again!")
            sys.exit(1)
        else:
            run_puzzle(new_puzzle, in_decrypt, get_answer)



import typing

from ...encryption_algorithms.simple import simple as se
from . import common as csc


class Simple:
    def __init__(self, **kwargs):
        self._scrypt_params = csc.scrypt_params(**kwargs)
        self._signature_params = csc.signature_params(**kwargs)

    def get_modules(self) -> typing.List[str]:
        return csc.MODULES

    def get_encrypt(self):
        return se.get_encrypt(self._scrypt_params, self._signature_params)

    def get_needed_objects(self):
        return csc.OBJECTS + [
            se.get_decrypt,
        ]

    def get_constants_str(
        self,
    ) -> str:
        _scrypt_params = csc.scrypt_params_to_code_str(**self._scrypt_params)
        _signature_params = "_SIGNATURE_PARAMS = " + repr(self._signature_params)
        decrypt: str = "_DECRYPT = get_decrypt(_SCRYPT_PARAMS, _SIGNATURE_PARAMS)"
        return "\n".join([_scrypt_params, _signature_params, decrypt])



import inspect
import typing

import black

from .puzzle_data_encryption import encrypt_data
from .configurators import configurators
from . import bytestr_utils


def question_answer_list_to_dict(qa_list: typing.List[str]):
    if len(qa_list) % 2 == 0:
        raise ValueError("The question/answer list must have odd length.")
    if len(qa_list) == 1:
        return {"str": qa_list[0]}
    return {
        "str": qa_list[0],
        "pass": qa_list[1],
        "rest": question_answer_list_to_dict(qa_list[2:]),
    }


def _create_str(in_encrypted_puzzle, configurator) -> str:
    advertisement = """# generated with puzzle-generator
#
# https://pypi.org/project/puzzle-generator/
# https://github.com/vil02/puzzle_generator/
"""
    modules: str = "\n".join("import " + _ for _ in configurator.get_modules()) + "\n"
    objects: str = "\n".join(
        inspect.getsource(_) for _ in configurator.get_needed_objects()
    )
    question = in_encrypted_puzzle[0]
    rest_str = bytestr_utils.bytes_to_bytestr(in_encrypted_puzzle[1])
    puzzle_data: str = f'_PUZZLE = ("{question}", bytestr_to_bytes("{rest_str}"))'
    call: str = "run_puzzle(_PUZZLE, _DECRYPT, input)"

    return (
        "\n".join(
            [
                advertisement,
                modules,
                'BYTEORDER: typing.Literal["little", "big"] = "little"',
                objects,
                puzzle_data,
                configurator.get_constants_str(),
                call,
            ]
        )
        + "\n"
    )


def create(qa_list: typing.List[str], **kwargs) -> str:
    puzzle = question_answer_list_to_dict(qa_list)
    configurator = configurators.get_configurator(**kwargs)
    encrypted_puzzle = encrypt_data(puzzle, configurator.get_encrypt())
    res = _create_str(encrypted_puzzle, configurator)
    return black.format_str(res, mode=black.FileMode())



import typing


BYTEORDER: typing.Literal["little", "big"] = "little"


def int_to_bytes(in_value: int) -> bytes:
    assert in_value >= 0  # nosec B101
    number_of_bytes = in_value.bit_length() // 8
    if 8 * number_of_bytes < in_value.bit_length():
        number_of_bytes += 1
    assert number_of_bytes <= 256  # nosec B101
    assert (
        number_of_bytes - 1 <= in_value.bit_length() // 8 <= number_of_bytes
    )  # nosec B101
    return number_of_bytes.to_bytes(length=1, byteorder=BYTEORDER) + in_value.to_bytes(
        length=number_of_bytes, byteorder=BYTEORDER
    )


def bytes_to_int(in_bytes: bytes) -> int:
    bytes_length = int(in_bytes[0])
    assert len(in_bytes) == bytes_length + 1  # nosec B101
    return int.from_bytes(in_bytes[1:], byteorder=BYTEORDER)


def join(in_str: str, in_bytes: bytes) -> bytes:
    str_as_bytes = in_str.encode()
    return int_to_bytes(len(str_as_bytes)) + str_as_bytes + in_bytes


def split(in_bytes: bytes) -> typing.Tuple[str, bytes]:
    end_of_length = int(in_bytes[0]) + 1
    str_as_bytes_len = bytes_to_int(in_bytes[:end_of_length])
    end_of_str = end_of_length + str_as_bytes_len
    str_as_bytes = in_bytes[end_of_length:end_of_str]
    res_bytes = in_bytes[end_of_str:]
    return str_as_bytes.decode(), res_bytes
